unit TestVectorMatrix;
interface
uses
  TestFramework, VectorMatrix;

type
  // Test methods for record TVector
  TestTVector = class(TTestCase)
  published
    procedure TestImplicit;
    procedure TestExplicit;
    procedure TestNegative;
    procedure TestEqual;
    procedure TestNotEqual;
    procedure TestAdd;
    procedure TestSubtract;
    procedure TestMultiply;
    procedure TestMultiply2;
  end;

  TestTMatrix = class(TTestCase)
  published
    procedure TestExplicit;
    procedure TestNegative;
    procedure TestEqual;
    procedure TestNotEqual;
    procedure TestAdd;
    procedure TestSubtract;
    procedure TestMultiply;
    procedure TestMultiply2;
    procedure TestMultiply3;
  end;

implementation

procedure TestTVector.TestImplicit;
var
  X: TVector;
  V: Vector;
begin
  SetLength(V,0);
  X := V;
  CheckEqualsString('()', String(X))
end;

procedure TestTVector.TestExplicit;
var
  X: TVector;
  V: Vector;
begin
  SetLength(V,3);
  V[0] := 1;
  V[1] := 2;
  V[2] := 4;
  X := V;
  CheckEqualsString('(1,2,4)', String(X))
end;

procedure TestTVector.TestNegative;
var
  X,Y: TVector;
  V: Vector;
begin
  SetLength(V,3);
  V[0] := 1;
  V[1] := -2;
  V[2] := 4;
  X := V;
  Y := -X;
  CheckEqualsString('(-1,2,-4)', String(Y))
end;

procedure TestTVector.TestEqual;
var
  X,Y: TVector;
  V: Vector;
begin
  SetLength(V,3);
  V[0] := 1;
  V[1] := 2;
  V[2] := 4;
  X := V;
  Y := V;
  Check(X = Y)
end;

procedure TestTVector.TestNotEqual;
var
  X,Y: TVector;
  V: Vector;
begin
  SetLength(V,3);
  V[0] := 1;
  V[1] := 2;
  V[2] := 4;
  X := V;
  Y := ---X;
  Check(X <> Y);
  CheckEqualsString('(-1,-2,-4)',String(Y))
end;

procedure TestTVector.TestAdd;
var
  X,Y,Z: TVector;
  V: Vector;
begin
  SetLength(V,3);
  V[0] := 1;
  V[1] := 2;
  V[2] := 4;
  X := V;
  V[0] := 3;
  V[1] := 5;
  V[2] := -7;
  Y := V;
  Z := X + Y;
  CheckEqualsString('(4,7,-3)', String(Z))
end;

procedure TestTVector.TestSubtract;
var
  X,Y,Z: TVector;
  V: Vector;
begin
  SetLength(V,3);
  V[0] := 1;
  V[1] := 2;
  V[2] := 4;
  X := V;
  V[0] := 3;
  V[1] := 5;
  V[2] := -7;
  Y := V;
  Z := X - Y;
  CheckEqualsString('(-2,-3,11)', String(Z))
end;

procedure TestTVector.TestMultiply;
var
  X,Y: TVector;
  V: Vector;
begin
  SetLength(V,3);
  V[0] := 1;
  V[1] := 2;
  V[2] := 4;
  X := V;
  Y := X * 3;
  CheckEqualsString('(3,6,12)', String(Y))
end;

procedure TestTVector.TestMultiply2;
var
  X,Y: TVector;
  M: TMatrix;
  V: Vector;
begin
  SetLength(V,2);
  V[0] := 1;
  V[1] := 2;
  X := V;
  Y := V;
  M := X * Y;
  CheckEqualsString('((1,2),(2,4))', String(M))
end;

{ TestTMatrix }

procedure TestTMatrix.TestExplicit;
var
  X,Y: TVector;
  M: TMatrix;
  V: Vector;
begin
  SetLength(V,3);
  V[0] := 1;
  V[1] := 2;
  V[2] := 3;
  X := V;
  Y := V;
  M := X * Y;
  CheckEqualsString('((1,2,3),(2,4,6),(3,6,9))', String(M))
end;

procedure TestTMatrix.TestNegative;
var
  X,Y: TVector;
  M,N: TMatrix;
  V: Vector;
begin
  SetLength(V,3);
  V[0] := 1;
  V[1] := 2;
  V[2] := 3;
  X := V;
  Y := V;
  M := X * Y;
  N := -M;
  CheckEqualsString('((-1,-2,-3),(-2,-4,-6),(-3,-6,-9))', String(N))
end;

procedure TestTMatrix.TestEqual;
var
  X,Y: TVector;
  M,N: TMatrix;
  V: Vector;
begin
  SetLength(V,3);
  V[0] := 1;
  V[1] := 2;
  V[2] := 3;
  X := V;
  Y := V;
  M := X * Y;
  N := --M;
  Check(M = N)
end;

procedure TestTMatrix.TestNotEqual;
var
  X,Y: TVector;
  M,N: TMatrix;
  V: Vector;
begin
  SetLength(V,3);
  V[0] := 1;
  V[1] := 2;
  V[2] := 3;
  X := V;
  Y := V;
  M := X * Y;
  N := -M;
  Check(M <> N)
end;

procedure TestTMatrix.TestAdd;
var
  X,Y: TVector;
  M,N,O: TMatrix;
  V: Vector;
begin
  SetLength(V,3);
  V[0] := 1;
  V[1] := 2;
  V[2] := 3;
  X := V;
  Y := V;
  M := X * Y;
  N := Y * X;
  O := M + N;
  CheckEqualsString('((2,4,6),(4,8,12),(6,12,18))', String(O))
end;

procedure TestTMatrix.TestSubtract;
var
  X,Y: TVector;
  M,N,O: TMatrix;
  V: Vector;
begin
  SetLength(V,3);
  V[0] := 1;
  V[1] := 2;
  V[2] := 3;
  X := V;
  Y := V;
  M := X * Y;
  N := -M;
  O := M - N;
  CheckEqualsString('((2,4,6),(4,8,12),(6,12,18))', String(O))
end;

procedure TestTMatrix.TestMultiply;
var
  X,Y: TVector;
  M: TMatrix;
  V,W: Vector;
begin
  SetLength(V,3);
  V[0] := 1;
  V[1] := 2;
  V[2] := 3;
  SetLength(W,2);
  W[0] := 1;
  W[1] := 2;
  X := V;
  Y := W;
  M := X * Y; // 3x2
  CheckEqualsString('((1,2),(2,4),(3,6))', String(M));
  M := M * 2;
  CheckEqualsString('((2,4),(4,8),(6,12))', String(M))
end;

procedure TestTMatrix.TestMultiply2;
var
  X,Y: TVector;
  M,N,O: TMatrix;
  V,W: Vector;
begin
  SetLength(V,3);
  V[0] := 1;
  V[1] := 2;
  V[2] := 3;
  SetLength(W,2);
  W[0] := 1;
  W[1] := 2;
  X := V;
  Y := W;
  M := X * Y; // 3x2
  CheckEqualsString('((1,2),(2,4),(3,6))', String(M));
  N := Y * X; // 2x3
  CheckEqualsString('((1,2,3),(2,4,6))', String(N));
  O := M * N; // 3x3
  CheckEqualsString('((5,10,15),(10,20,30),(15,30,45))', String(O));
  O := N * M; // 2x2
  CheckEqualsString('((14,28),(28,56))', String(O))
end;

procedure TestTMatrix.TestMultiply3;
var
  X,Y: TVector;
  M,N,O: TMatrix;
  V,W: Vector;
begin
  SetLength(V,3);
  V[0] := 1;
  V[1] := 2;
  V[2] := 3;
  SetLength(W,1);
  W[0] := 1;
  X := V;
  Y := W;
  M := X * Y; // 3x1 matrix = row vector
  CheckEqualsString('((1),(2),(3))', String(M));
  N := X * X; // 3x3
  CheckEqualsString('((1,2,3),(2,4,6),(3,6,9))', String(N));
  O := N * M; // 3x3 * 3x1 = 3x1
  CheckEqualsString('((14),(28),(42))', String(O));
end;

initialization
  // Register any test cases with the test runner
  RegisterTest(TestTVector.Suite);
  RegisterTest(TestTMatrix.Suite);
end.

